package net.lemnik.eodsql;

import java.util.Map;

/**
 * This interface can be used as a mix-in for SQL queries and statements created
 * dynamically at runtime. Usage is like this:
 * 
 * <pre>
 * <code>
 * public interface MyQuery extends BaseQuery, DynamicQuery {
 *     @Select(sql = "SELECT * FROM persons")
 *     public DataSet&lt;Address&gt; list();
 * 
 *     @Select(sql = "SELECT * FROM persons")
 *     public DataSet&lt;Map&lt;String,Object&gt;&gt; listMap();
 * }
 * ...
 * MyQuery query = QueryTool.getQuery(conn, MyQuery.class);
 * DataSet&lt;Map&lt;String,Object&gt;&gt; data = query.select("SELECT * FROM persons WHERE last_name = 'Smith'");
 * </code>
 * </pre>
 * <p>
 * <code>DynamicQuery</code> can be mixed-in to {@link BaseQuery} and {@link TransactionQuery}. 
 * The interfaces {@link DynamicBaseQuery} and {@link DynamicTransactionQuery} can be used for that.
 * 
 * @author Bernd Rinn
 */
public interface DynamicQuery {

    /**
     * Performs a SQL query. The returned <code>DataSet</code> is connected and
     * updateable.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The result set as <code>DataSet</code>; each row is represented
     *         as one <code>Map&lt;String,Object&gt;</code>.
     */
    DataSet<Map<String, Object>> select(final String query,
            final Object... parameters) throws InvalidDataTypeException,
            InvalidQueryException, EoDException;

    /**
     * Performs a SQL query.
     * 
     * @param readOnly
     *            If <code>true</code>, the returned <code>DataSet</code> will
     *            not be updateable.
     * @param disconnected
     *            If <code>true</code>, the returned <code>DataSet</code> will
     *            be disconnected.
     * @param fetchSize
     *            The fetchSize given to the JDBC driver.
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The result set as <code>DataSet</code>; each row is represented
     *         as one <code>Map&lt;String,Object&gt;</code>.
     */
    DataSet<Map<String, Object>> select(final boolean readOnly,
            final boolean disconnected, final int fetchSize,
            final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Performs a SQL query. The returned <code>DataSet</code> is connected and
     * updateable.
     * 
     * @param type
     *            The Java type to return one rows in the returned
     *            <code>DataSet</code>.
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The result set as <code>DataSet</code>; each row is represented
     *         as one <code>Map&lt;String,Object&gt;</code>.
     */
    <T> DataSet<T> select(final Class<T> type, final String query,
            final Object... parameters) throws InvalidDataTypeException,
            InvalidQueryException, EoDException;

    /**
     * Performs a SQL query.
     * 
     * @param type
     *            The Java type to return one rows in the returned
     *            <code>DataSet</code>.
     * @param readOnly
     *            If <code>true</code>, the returned <code>DataSet</code> will
     *            not be updateable.
     * @param disconnected
     *            If <code>true</code>, the returned <code>DataSet</code> will
     *            be disconnected.
     * @param fetchSize
     *            The fetchSize given to the JDBC driver.
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The result set as <code>DataSet</code>; each row is represented
     *         as one <code>Map&lt;String,Object&gt;</code>.
     */
    <T> DataSet<T> select(final Class<T> type, final boolean readOnly,
            final boolean disconnected, final int fetchSize,
            final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Executes a SQL statement.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The number of rows updated by the SQL statement, or -1 if not
     *         applicable. <b>Note:</b> Not all JDBC drivers support this
     *         cleanly.
     */
    int update(final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Executes a SQL statement as a batch for all parameter values provided.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template. At least
     *            one of the parameters needs to be an array or
     *            <code>Collection</code>. If multiple parameters are arrays or
     *            <code>Collection</code>, all of them need to have the same
     *            size.
     * 
     * @return The number of rows updated by the SQL statement, or -1 if not
     *         applicable. <b>Note:</b> Not all JDBC drivers support this
     *         cleanly.
     */
    int batchUpdate(final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Executes a SQL statement. Supposed to be used for INSERT statements with
     * an automatically generated integer key.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The automatically generated key. <b>Note:</b> Not all JDBC
     *         drivers support this cleanly.
     */
    long insert(final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Executes a SQL statement. Supposed to be used for INSERT statements with
     * one or more automatically generated keys.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template.
     * 
     * @return The automatically generated keys. <b>Note:</b> Not all JDBC
     *         drivers support this cleanly and it is very driver-dependent what
     *         keys are present in the returned map.
     */
    Map<String, Object> insertMultiKeys(final String query,
            final Object... parameters) throws InvalidDataTypeException,
            InvalidQueryException, EoDException;

    /**
     * Executes a SQL statement as a batch for all parameter values provided.
     * Supposed to be used for INSERT statements with an automatically generated
     * integer key.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template. At least
     *            one of the parameters needs to be an array or
     *            <code>Collection</code>. If multiple parameters are arrays or
     *            <code>Collection</code>, all of them need to have the same
     *            size.
     * 
     * @return The automatically generated key for each element of the batch.
     *         <b>Note:</b> Not all JDBC drivers support this cleanly.
     */
    long[] batchInsert(final String query, final Object... parameters)
            throws InvalidDataTypeException, InvalidQueryException,
            EoDException;

    /**
     * Executes a SQL statement as a batch for all parameter values provided.
     * Supposed to be used for INSERT statements with one or more automatically
     * generated keys.
     * 
     * @param query
     *            The SQL query template.
     * @param parameters
     *            The parameters to fill into the SQL query template. At least
     *            one of the parameters needs to be an array or
     *            <code>Collection</code>. If multiple parameters are arrays or
     *            <code>Collection</code>, all of them need to have the same
     *            size.
     * 
     * @return The automatically generated keys for each element of the batch.
     *         <b>Note:</b> Not all JDBC drivers support this cleanly and it is
     *         very driver-dependent what keys are present in the returned map.
     */
    Map<String, Object>[] batchInsertMultiKeys(final String query,
            final Object... parameters) throws InvalidDataTypeException,
            InvalidQueryException, EoDException;
}
