package net.lemnik.eodsql.impl;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.text.ParseException;

import net.lemnik.eodsql.ArrayDataSetCache;
import net.lemnik.eodsql.DataSetCache;
import net.lemnik.eodsql.Select;
import net.lemnik.eodsql.TypeMapper;
import net.lemnik.eodsql.spi.MethodImplementation;
import net.lemnik.eodsql.spi.util.DataObjectBinding;
import net.lemnik.eodsql.spi.util.NoDataObjectBinding;

/**
 * A {@link DynamicParameterFactory} for SELECT query method invocations.
 *
 * @author Bernd Rinn
 */
class SelectDynamicParameterFactory implements DynamicParameterFactory {
    
    private final Class<?> baseReturnType;
    
    private final int fetchSize;
    
    private final boolean disconnected;
    
    private final boolean readOnly;

    private final int sqlParamPos;
    
    private final int fetchSizeParamPos;
    
    private final int readOnlyParamPos;
    
    private final int disconnectedParamPos;
    
    private final int baseReturnTypeParamPos;
    
    SelectDynamicParameterFactory(final Class<?> baseReturnType, final int baseReturnTypeParamPos, 
            final int sqlParamPos, final int fetchSizeParamPos, 
            final int readOnlyParamPos, final int disconnectedParamPos) {
        this.baseReturnType = baseReturnType;
        this.fetchSize = 0;
        this.disconnected = false;
        this.readOnly = false;
        this.baseReturnTypeParamPos = baseReturnTypeParamPos;
        this.sqlParamPos = sqlParamPos;
        this.fetchSizeParamPos = fetchSizeParamPos;
        this.readOnlyParamPos = readOnlyParamPos;
        this.disconnectedParamPos = disconnectedParamPos;
    }
    
    SelectDynamicParameterFactory(final Class<?> baseReturnType, final int baseReturnTypeParamPos, 
            final int sqlParamPos, final int fetchSize, final boolean disconnected, final boolean readOnly) {
        this.baseReturnType = baseReturnType;
        this.fetchSize = fetchSize;
        this.disconnected = disconnected;
        this.readOnly = readOnly;
        this.baseReturnTypeParamPos = baseReturnTypeParamPos;
        this.sqlParamPos = sqlParamPos;
        this.fetchSizeParamPos = -1;
        this.readOnlyParamPos = -1;
        this.disconnectedParamPos = -1;
    }

    public Annotation createAnnotation(final Object[] args) {
        return new Select() {

            public String value() {
                return "";
            }

            public String sql() {
                return getSql(args);
            }

            public boolean disconnected() {
                return (disconnectedParamPos < 0) ? disconnected : ((Boolean) args[disconnectedParamPos]);
            }

            public boolean rubberstamp() {
                return false;
            }

            public boolean readOnly() {
                return (readOnlyParamPos < 0) ? readOnly : ((Boolean) args[readOnlyParamPos]);
            }

            public Class<? extends DataSetCache> cache() {
                return ArrayDataSetCache.class;
            }

            public Class<? extends Annotation> annotationType() {
                return Select.class;
            }

            public int fetchSize() {
                return (fetchSizeParamPos < 0) ? fetchSize : ((Integer) args[fetchSizeParamPos]);
            }

            public int into() {
                return 0;
            }

            public Class<? extends DataObjectBinding> resultSetBinding()
            {
                return NoDataObjectBinding.class;
            }

            @SuppressWarnings("unchecked")
                public Class<? extends TypeMapper>[] parameterBindings()
                {
                    return new Class[0];
                }

            @Override
            public int hashCode() {
                int hash = "Select".hashCode();
                hash = 37 * hash + sql().hashCode();
                hash = 37 * hash + (disconnected() ? 1 : 0);
                hash = 37 * hash + (readOnly() ? 1 : 0);
                hash = 37 * hash + fetchSize();
                return hash;
            }
            
            @Override
            public boolean equals(Object obj) {
                if (obj instanceof Select == false) {
                    return false;
                }
                Select other = (Select) obj;
                return sql().equals(other.sql()) && disconnected() == other.disconnected() 
                        && readOnly() == other.readOnly() && fetchSize() == other.fetchSize();
            }
        };
        
    }
    
    public String getSql(Object[] args) {
        return (String) args[sqlParamPos];
    }
    
    public MethodImplementation<? extends Annotation> createMethodImplementation(Method method,
            Annotation annotation, Class[] queryParamTypes, Type returnType, Object[] args) throws ParseException {
        return new SelectMethodImplementation((Select) annotation, queryParamTypes, returnType);
    }
    
    public Class createBaseReturnType(final Object[] args) {
        return (baseReturnTypeParamPos < 0) ? baseReturnType : (Class) args[baseReturnTypeParamPos];
    }

    public boolean isSingleRow() {
        return false;
    }
    
}